@@ -2,7 +2,7 @@
  * $Id: connection.c,v 1.98 2005/04/12 19:34:35 nohar Exp $
  *
  * This file is part of the bip project
- * Copyright (C) 2004 2005 Arnaud Cornet and Loc Gomez
+ * Copyright (C) 2004 2005 Arnaud Cornet and Loïc Gomez
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@ int link_add_untrusted(void *ls, X509 *cert);
 
 static int connection_timedout(connection_t *cn);
 static int socket_set_nonblock(int s);
+static void connection_connected(connection_t *c);
 
 struct connecting_data
 {
@@ -50,9 +51,9 @@ static void connecting_data_free(struct connecting_data *t)
 
 void connection_close(connection_t *cn)
 {
-	mylog(LOG_DEBUGVERB, "Connection close asked. FD:%d ",
+	mylog(LOG_DEBUG, "Connection close asked. FD:%d ",
 			(long)cn->handle);
-	if (cn->connected != CONN_DISCONN) {
+	if (cn->connected != CONN_DISCONN && cn->connected != CONN_ERROR) {
 		cn->connected = CONN_DISCONN;
 		if (close(cn->handle) == -1)
 			mylog(LOG_WARN, "Error on socket close: %s",
@@ -96,6 +97,14 @@ void connection_free(connection_t *cn)
 		}
 	}
 #endif
+	if (cn->localip) {
+		free(cn->localip);
+		cn->localip = NULL;
+	}
+	if (cn->remoteip) {
+		free(cn->remoteip);
+		cn->remoteip = NULL;
+	}
 	free(cn);
 }
 
@@ -144,6 +153,7 @@ static void connect_trynext(connection_t *cn)
 			connecting_data_free(cn->connecting_data);
 			cn->connecting_data = NULL;
 			cn->connected = cn->ssl ? CONN_NEED_SSLIZE : CONN_OK;
+			connection_connected(cn);
 			return;
 		}
 
@@ -168,7 +178,7 @@ static X509 *mySSL_get_cert(SSL *ssl)
 	X509 *cert;
 
 	if (!ssl) {
-		mylog(LOG_DEBUGVERB, "mySSL_get_cert() No SSL context");
+		mylog(LOG_ERROR, "mySSL_get_cert() No SSL context");
 		return NULL;
 	}
 	cert = SSL_get_peer_certificate(ssl);
@@ -187,8 +197,7 @@ static int _write_socket_SSL(connection_t *cn, char* message)
 	if (!cn->client && cn->cert == NULL) {
 		cn->cert = mySSL_get_cert(cn->ssl_h);
 		if (cn->cert == NULL) {
-			mylog(LOG_DEBUGVERB,
-					"No certificate in SSL write_socket");
+			mylog(LOG_ERROR, "No certificate in SSL write_socket");
 			return WRITE_ERROR;
 		}
 	}
@@ -201,7 +210,7 @@ static int _write_socket_SSL(connection_t *cn, char* message)
 				|| err == SSL_ERROR_WANT_ACCEPT)
 			return WRITE_KEEP;
 		if (cn_is_connected(cn)) {
-			mylog(LOG_DEBUGVERB, "fd %d: Connection error",
+			mylog(LOG_ERROR, "fd %d: Connection error",
 					cn->handle);
 			cn->connected = CONN_ERROR;
 		}
@@ -261,11 +270,11 @@ static int _write_socket(connection_t *cn, char *message)
 		 */
 
 		if (cn_is_connected(cn)) {
-			mylog(LOG_DEBUG, "write(fd %d) : %s", cn->handle,
+			mylog(LOG_DEBUGVERB, "write(fd %d) : %s", cn->handle,
 					strerror(errno));
 			cn->connected = CONN_ERROR;
 		}
-		mylog(LOG_DEBUG, "write : %s", strerror(errno));
+		mylog(LOG_DEBUGVERB, "write : %s", strerror(errno));
 		return WRITE_ERROR;
 	}
 	mylog(LOG_DEBUGVERB, "%d/%d bytes sent !", tcount, size);
@@ -396,7 +405,7 @@ static int read_socket_SSL(connection_t *cn)
 				|| err == SSL_ERROR_WANT_ACCEPT)
 			return 0;
 		if (cn_is_connected(cn)) {
-			mylog(LOG_DEBUGTOOMUCH, "fd %d: Connection error",
+			mylog(LOG_ERROR, "fd %d: Connection error",
 					cn->handle);
 			cn->connected = CONN_ERROR;
 		}
@@ -408,7 +417,7 @@ static int read_socket_SSL(connection_t *cn)
 				|| err == SSL_ERROR_WANT_ACCEPT)
 			return 0;*/
 		if (cn_is_connected(cn)) {
-			mylog(LOG_DEBUGTOOMUCH, "fd %d: Connection lost",
+			mylog(LOG_ERROR, "fd %d: Connection lost",
 					cn->handle);
 			connection_close(cn);
 		}
@@ -434,20 +443,16 @@ static int read_socket(connection_t *cn)
 		if (errno == EAGAIN || errno == EINTR || errno == EINPROGRESS)
 			return 0;
 		if (cn_is_connected(cn)) {
-			mylog(LOG_DEBUGVERB, "fd %d: Connection error",
-					cn->handle);
-			mylog(LOG_DEBUGTOOMUCH, "fd %d: read() %s", cn->handle,
-					strerror(errno));
+			mylog(LOG_ERROR, "read(fd=%d): Connection error: %s",
+					cn->handle, strerror(errno));
 			cn->connected = CONN_ERROR;
 		}
 		return 1;
 	} else if (count == 0) {
 		if (cn_is_connected(cn)) {
-			mylog(LOG_DEBUGVERB, "fd %d: Connection lost",
-					cn->handle);
+			mylog(LOG_ERROR, "read(fd=%d): Connection lost: %s",
+					cn->handle, strerror(errno));
 			connection_close(cn);
-			mylog(LOG_DEBUGTOOMUCH, "fd %d: read() %s", cn->handle,
-					strerror(errno));
 		}
 		return 1;
 	}
@@ -543,7 +548,7 @@ static int check_event_except(fd_set *fds, connection_t *cn)
 		return 0;
 
 	if (cn_is_in_error(cn)) {
-		mylog(LOG_DEBUGVERB, "Error on fd %d (state %d)",
+		mylog(LOG_ERROR, "Error on fd %d (state %d)",
 				cn->handle, cn->connected);
 		return 1;
 	}
@@ -561,7 +566,7 @@ static int check_event_read(fd_set *fds, connection_t *cn)
 	int ret;
 
 	if (cn_is_in_error(cn)) {
-		mylog(LOG_DEBUGVERB, "Error on fd %d (state %d)",
+		mylog(LOG_ERROR, "Error on fd %d (state %d)",
 				cn->handle, cn->connected);
 		return 1;
 	}
@@ -569,7 +574,7 @@ static int check_event_read(fd_set *fds, connection_t *cn)
 	if (!FD_ISSET(cn->handle, fds))
 		return 0;
 
-	mylog(LOG_DEBUGVERB, "Read positif on fd %d (state %d)", cn->handle,
+	mylog(LOG_DEBUGTOOMUCH, "Read positive on fd %d (state %d)", cn->handle,
 			cn->connected);
 
 	/* notify caller to make it check for a new client */
@@ -584,10 +589,10 @@ static int check_event_read(fd_set *fds, connection_t *cn)
 		ret = read_socket(cn);
 
 	if (ret) {
-		mylog(LOG_DEBUGVERB, "Error while reading on fd %d",
+		mylog(LOG_ERROR, "Error while reading on fd %d",
 				cn->handle);
- 		return 1;
- 	}
+		return 1;
+	}
 
 	if (!cn->incoming_lines)
 		cn->incoming_lines = list_new(NULL);
@@ -595,15 +600,27 @@ static int check_event_read(fd_set *fds, connection_t *cn)
 	if (list_is_empty(cn->incoming_lines))
 		return 0;
 
-	mylog(LOG_DEBUGVERB, "newlines on fd %d (state %d)", cn->handle,
+	mylog(LOG_DEBUGTOOMUCH, "newlines on fd %d (state %d)", cn->handle,
 			cn->connected);
 	return 1;
 }
 
+static void connection_connected(connection_t *c)
+{
+	if (c->localip)
+		free(c->localip);
+	c->localip = connection_localip(c);
+	c->localport = connection_localport(c);
+	if (c->remoteip)
+		free(c->remoteip);
+	c->remoteip = connection_remoteip(c);
+	c->remoteport = connection_remoteport(c);
+}
+
 static int check_event_write(fd_set *fds, connection_t *cn, int *nc)
 {
 	if (cn_is_in_error(cn)) {
-		mylog(LOG_DEBUGVERB, "Error on fd %d (state %d)",
+		mylog(LOG_ERROR, "Error on fd %d (state %d)",
 				cn->handle, cn->connected);
 		return 1;
 	}
@@ -612,13 +629,13 @@ static int check_event_write(fd_set *fds, connection_t *cn, int *nc)
 		if (cn_is_connected(cn))
 			return 0;
 
-		mylog(LOG_DEBUGVERB, "New socket still not connected (%d)",
+		mylog(LOG_DEBUG, "New socket still not connected (%d)",
 				cn->handle);
 		/* check timeout (handles connect_trynext) */
 		return connection_timedout(cn);
 	}
 
-	mylog(LOG_DEBUGVERB, "Write positif on fd %d (state %d)",
+	mylog(LOG_DEBUGTOOMUCH, "Write positive on fd %d (state %d)",
 			cn->handle, cn->connected);
 
 	if (cn_is_new(cn)) {
@@ -629,7 +646,7 @@ static int check_event_write(fd_set *fds, connection_t *cn, int *nc)
 				(void *)&err, &errSize);
 
 		if (err2 < 0) {
-			mylog(LOG_DEBUGVERB, "fd:%d getsockopt error: %s",
+			mylog(LOG_ERROR, "fd:%d getsockopt error: %s",
 					cn->handle, strerror(errno));
 			if (cn->connecting_data) {
 				close(cn->handle);
@@ -651,6 +668,7 @@ static int check_event_write(fd_set *fds, connection_t *cn, int *nc)
 			}
 #endif
 			cn->connected = CONN_OK;
+			connection_connected(cn);
 			*nc = 1;
 			mylog(LOG_DEBUG, "fd:%d Connection established !",
 					cn->handle);
@@ -741,7 +759,7 @@ list_t *wait_event(list_t *cn_list, int *msec, int *nc)
 	struct timeval btv, etv;
 	*nc = 0;
 
-	cn_newdata = list_new(NULL);
+	cn_newdata = list_new(list_ptr_cmp);
 	FD_ZERO(&fds_read);
 	FD_ZERO(&fds_write);
 	FD_ZERO(&fds_except);
@@ -752,7 +770,7 @@ list_t *wait_event(list_t *cn_list, int *msec, int *nc)
 
 		mylog(LOG_DEBUGTOOMUCH, "I've seen socket %d !", cn->handle);
 		if (cn->connected == CONN_DISCONN) {
-			list_add_first(cn_newdata, cn);
+			list_add_first_uniq(cn_newdata, cn);
 			continue;
 		}
 
@@ -760,7 +778,7 @@ list_t *wait_event(list_t *cn_list, int *msec, int *nc)
 		 * This shouldn't happen ! just in case...
 		 */
 		if (cn->handle < 0) {
-			mylog(LOG_DEBUG, "wait_event invalid socket %d",
+			mylog(LOG_WARN, "wait_event invalid socket %d",
 					cn->handle);
 			if (cn_is_connected(cn))
 				cn->connected = CONN_ERROR;
@@ -778,7 +796,7 @@ list_t *wait_event(list_t *cn_list, int *msec, int *nc)
 		 */
 		if (cn_is_connected(cn)) {
 			FD_SET(cn->handle, &fds_read);
-			mylog(LOG_DEBUGVERB, "Test read on fd %d %d:%d",
+			mylog(LOG_DEBUGTOOMUCH, "Test read on fd %d %d:%d",
 					cn->handle, cn->connected,
 					cn_is_connected(cn));
 		}
@@ -789,7 +807,7 @@ list_t *wait_event(list_t *cn_list, int *msec, int *nc)
 
 		if (!cn_is_connected(cn) || cn_want_write(cn)) {
 			FD_SET(cn->handle, &fds_write);
-			mylog(LOG_DEBUGVERB, "Test write on fd %d %d:%d",
+			mylog(LOG_DEBUGTOOMUCH, "Test write on fd %d %d:%d",
 					cn->handle, cn->connected,
 					cn_is_connected(cn));
 		}
@@ -805,7 +823,7 @@ list_t *wait_event(list_t *cn_list, int *msec, int *nc)
 	tv.tv_sec = *msec / 1000;
 	tv.tv_usec = (*msec % 1000) * 1000;
 	gettimeofday(&btv, NULL);
-	mylog(LOG_DEBUGVERB, "msec: %d, sec: %d, usec: %d", *msec, tv.tv_sec,
+	mylog(LOG_DEBUGTOOMUCH, "msec: %d, sec: %d, usec: %d", *msec, tv.tv_sec,
 			tv.tv_usec);
 	err = select(maxfd + 1, &fds_read, &fds_write, &fds_except, &tv);
 	gettimeofday(&etv, NULL);
@@ -818,7 +836,7 @@ list_t *wait_event(list_t *cn_list, int *msec, int *nc)
 	/* in case we go forward in time */
 	if (*msec < 0)
 		*msec = 0;
-	mylog(LOG_DEBUGVERB, "msec: %d, sec: %d, usec: %d", *msec, tv.tv_sec,
+	mylog(LOG_DEBUGTOOMUCH, "msec: %d, sec: %d, usec: %d", *msec, tv.tv_sec,
 			tv.tv_usec);
 	if (err < 0) {
 		if (errno == EINTR)
@@ -835,23 +853,23 @@ list_t *wait_event(list_t *cn_list, int *msec, int *nc)
 		int toadd = 0;
 
 		if (check_event_except(&fds_except, cn)) {
-			mylog(LOG_DEBUGVERB, "Notify on FD %d (state %d)",
+			mylog(LOG_DEBUGTOOMUCH, "Notify on FD %d (state %d)",
 					cn->handle, cn->connected);
-			list_add_first(cn_newdata, cn);
+			list_add_first_uniq(cn_newdata, cn);
 			continue;
- 		}
+		}
 		if (check_event_write(&fds_write, cn, nc)) {
 			if (cn_is_in_error(cn))
 				toadd = 1;
 		}
 
 		if (check_event_read(&fds_read, cn)) {
-			mylog(LOG_DEBUGVERB, "Notify on FD %d (state %d)",
+			mylog(LOG_DEBUGTOOMUCH, "Notify on FD %d (state %d)",
 					cn->handle, cn->connected);
 			toadd = 1;
 		}
 		if (toadd)
-			list_add_first(cn_newdata, cn);
+			list_add_first_uniq(cn_newdata, cn);
 	}
 	return cn_newdata;
 }
@@ -878,17 +896,18 @@ static void create_socket(char *dsthostname, char *dstport, char *srchostname,
 
 	err = getaddrinfo(dsthostname, dstport, &hint, &cdata->dst);
 	if (err) {
-		mylog(LOG_DEBUG, "getaddrinfo(dst): %s", gai_strerror(err));
+		mylog(LOG_ERROR, "getaddrinfo(%s): %s", dsthostname,
+				gai_strerror(err));
 		connecting_data_free(cdata);
 		cdata = NULL;
 		return;
- 	}
+	}
 
 	if (srchostname || srcport) {
 		if ((err = getaddrinfo(srchostname, srcport, &hint,
 						&cdata->src))) {
 			/* not fatal ? maybe a config option is needed */
-			mylog(LOG_DEBUG, "getaddrinfo(src): %s",
+			mylog(LOG_ERROR, "getaddrinfo(src): %s",
 					gai_strerror(err));
 			cdata->src = NULL;
 		}
@@ -909,7 +928,7 @@ static void create_listening_socket(char *hostname, char *port,
 	struct addrinfo *res, *cur;
 	struct addrinfo hint = {
 		.ai_flags = AI_PASSIVE,
-		.ai_family = PF_UNSPEC,
+		.ai_family = AF_UNSPEC,
 		.ai_socktype = SOCK_STREAM,
 		.ai_protocol = 0,
 
@@ -923,9 +942,9 @@ static void create_listening_socket(char *hostname, char *port,
 
 	err = getaddrinfo(hostname, port, &hint, &res);
 	if (err) {
-		mylog(LOG_DEBUG, "getaddrinfo(): %s", gai_strerror(err));
+		mylog(LOG_ERROR, "getaddrinfo(): %s", gai_strerror(err));
 		return;
- 	}
+	}
 
 	for (cur = res ; cur ; cur = cur->ai_next) {
 		if ((cn->handle = socket(cur->ai_family, cur->ai_socktype,
@@ -976,7 +995,7 @@ static connection_t *connection_init(int anti_flood, int ssl, int timeout,
 	char *incoming;
 	list_t *outgoing;
 
-	conn = (connection_t*)malloc(sizeof(connection_t));
+	conn = (connection_t *)calloc(sizeof(connection_t), 1);
 	incoming = (char*)malloc(sizeof(char) * CONN_BUFFER_SIZE);
 	outgoing = list_new(NULL);
 
@@ -1000,7 +1019,6 @@ static connection_t *connection_init(int anti_flood, int ssl, int timeout,
 	conn->ssl_ctx_h = NULL;
 	conn->ssl_h = NULL;
 	conn->cert = NULL;
-	conn->ssl_check_store = NULL;
 	conn->ssl_check_mode = SSL_CHECK_NONE;
 #endif
 	conn->connected = CONN_NEW;
@@ -1016,8 +1034,10 @@ connection_t *accept_new(connection_t *cn)
 
 	mylog(LOG_DEBUG, "Trying to accept new client on %d", cn->handle);
 	err = accept(cn->handle, &sa, &sa_len);
-	if (err < 0)
+	if (err < 0) {
+		mylog(LOG_ERROR, "accept failed: %s", strerror(errno));
 		return NULL;
+	}
 	socket_set_nonblock(err);
 
 	conn = connection_init(cn->anti_flood, cn->ssl, cn->timeout, 0);
@@ -1032,11 +1052,19 @@ connection_t *accept_new(connection_t *cn)
 					"accepted connections. "
 					"Initializing...");
 			if (!(sslctx = SSL_init_context())) {
-				mylog(LOG_DEBUG, "SSL context initialization "
+				mylog(LOG_ERROR, "SSL context initialization "
 						"failed");
 				connection_free(conn);
 				return NULL;
 			}
+			if (!SSL_CTX_use_certificate_chain_file(sslctx,
+						conf_ssl_certfile))
+				mylog(LOG_WARN, "SSL: Unable to load "
+						"certificate file");
+			if (!SSL_CTX_use_PrivateKey_file(sslctx,
+						conf_ssl_certfile,
+						SSL_FILETYPE_PEM))
+				mylog(LOG_WARN, "SSL: Unable to load key file");
 		}
 
 		conn->ssl_h = SSL_new(sslctx);
@@ -1108,7 +1136,7 @@ static SSL_CTX *SSL_init_context(void)
 		do {
 			ret = read(fd, buf, 1024);
 			if (ret <= 0) {
-				mylog(LOG_DEBUG,"/dev/random: %s",
+				mylog(LOG_ERROR,"/dev/random: %s",
 						strerror(errno));
 				goto prng_end;
 			}
@@ -1134,14 +1162,8 @@ prng_end:
 
 	/* allocated by function */
 	ctx = SSL_CTX_new(SSLv23_method());
-	if (!SSL_CTX_use_certificate_chain_file(ctx,conf_ssl_certfile))
-		mylog(LOG_DEBUG, "SSL: Unable to load certificate file");
-	if (!SSL_CTX_use_PrivateKey_file(ctx, conf_ssl_certfile,
-				SSL_FILETYPE_PEM))
-		mylog(LOG_DEBUG, "SSL: Unable to load key file");
-
 	SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_BOTH);
-	SSL_CTX_set_timeout(ctx,(long)60);
+	SSL_CTX_set_timeout(ctx, (long)60);
 	SSL_CTX_set_options(ctx, SSL_OP_ALL);
 
 	return ctx;
@@ -1181,14 +1203,20 @@ static int bip_ssl_verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 			(err == X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY ||
 			 err == X509_V_ERR_CERT_UNTRUSTED ||
 			 err == X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE ||
-			 err == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT)) {
+			 err == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT ||
+			 err == X509_V_ERR_CERT_HAS_EXPIRED)) {
 
 		if (X509_STORE_get_by_subject(ctx, X509_LU_X509,
 				X509_get_subject_name(err_cert), &xobj) > 0 &&
 				!X509_cmp(xobj.data.x509, err_cert)) {
 
-			mylog(LOG_INFO, "Basic mode; peer certificate found "
-					"in store, accepting it!");
+			if (err == X509_V_ERR_CERT_HAS_EXPIRED)
+				mylog(LOG_INFO, "Basic mode; Accepting "
+						"*expired* peer certificate "
+						"found in store.");
+			else
+				mylog(LOG_INFO, "Basic mode; Accepting peer "
+					"certificate found in store.");
 
 			result = 1;
 			err = X509_V_OK;
@@ -1220,12 +1248,12 @@ static int SSLize(connection_t *cn, int *nc)
 		return 1;
 
 	if (cn->listening) {
-		mylog(LOG_DEBUG, "Can't use SSL with listening socket.");
+		mylog(LOG_ERROR, "Can't use SSL with listening socket.");
 		return 0;
 	}
 
 	if (!SSL_set_fd(cn->ssl_h, cn->handle)) {
-		mylog(LOG_DEBUG, "unable to associate FD to SSL structure");
+		mylog(LOG_ERROR, "unable to associate FD to SSL structure");
 		cn->connected = CONN_ERROR;
 		return 1;
 	}
@@ -1237,7 +1265,7 @@ static int SSLize(connection_t *cn, int *nc)
 
 	err2 = SSL_get_error(cn->ssl_h, err);
 	ERR_print_errors(errbio);
-	
+
 	if (err2 == SSL_ERROR_NONE) {
 		SSL_CIPHER *cipher;
 		char buf[128];
@@ -1251,6 +1279,7 @@ static int SSLize(connection_t *cn, int *nc)
 		mylog(LOG_DEBUG, "Negociated cyphers: %s", buf);
 
 		cn->connected = CONN_OK;
+		connection_connected(cn);
 		*nc = 1;
 		return 0;
 	}
@@ -1283,7 +1312,7 @@ static int SSLize(connection_t *cn, int *nc)
 	}
 	/* From now on, we are on error, thus we return 1 to check timeout */
 	if (err2 == SSL_ERROR_ZERO_RETURN || err2 == SSL_ERROR_SSL) {
-		mylog(LOG_DEBUG, "Error in SSL handshake.");
+		mylog(LOG_ERROR, "Error in SSL handshake.");
 		cn->connected = CONN_ERROR;
 		return 1;
 	}
@@ -1294,18 +1323,18 @@ static int SSLize(connection_t *cn, int *nc)
 
 static connection_t *_connection_new_SSL(char *dsthostname, char *dstport,
 		char *srchostname, char *srcport, int check_mode,
-		char *check_store, int timeout)
+		char *check_store, char *ssl_client_certfile, int timeout)
 {
 	connection_t *conn;
 
 	conn = connection_init(1, 1, timeout, 0);
 	if (!(conn->ssl_ctx_h = SSL_init_context())) {
-		mylog(LOG_DEBUG, "SSL context initialization failed");
+		mylog(LOG_ERROR, "SSL context initialization failed");
 		return conn;
 	}
+
 	conn->cert = NULL;
 	conn->ssl_check_mode = check_mode;
-	conn->ssl_check_store = check_store;
 
 	switch (conn->ssl_check_mode) {
 	case SSL_CHECK_BASIC:
@@ -1342,15 +1371,27 @@ static connection_t *_connection_new_SSL(char *dsthostname, char *dstport,
 		fatal("Unknown SSL cert check mode.");
 	}
 
+	if (ssl_client_certfile) {
+		if (!SSL_CTX_use_certificate_chain_file(conn->ssl_ctx_h,
+					ssl_client_certfile))
+			mylog(LOG_WARN, "SSL: Unable to load certificate file");
+		else if (!SSL_CTX_use_PrivateKey_file(conn->ssl_ctx_h,
+					ssl_client_certfile, SSL_FILETYPE_PEM))
+			mylog(LOG_WARN, "SSL: Unable to load key file");
+		else
+			mylog(LOG_INFO, "SSL: using %s pem file as client SSL "
+					"certificate", ssl_client_certfile);
+	}
+
 	conn->ssl_h = SSL_new(conn->ssl_ctx_h);
 	if (conn->ssl_h == NULL) {
-		mylog(LOG_DEBUG, "Unable to allocate SSL structures");
+		mylog(LOG_ERROR, "Unable to allocate SSL structures");
 		return conn;
 	}
 	/* ys: useless as long as we have a context by connection
 	if (sslctx->session_cache_head)
 		if (!SSL_set_session(conn->ssl_h, sslctx->session_cache_head))
-			mylog(LOG_DEBUG, "unable to set SSL session id to"
+			mylog(LOG_ERROR, "unable to set SSL session id to"
 					" most recent used");
 	*/
 	SSL_set_connect_state(conn->ssl_h);
@@ -1366,7 +1407,7 @@ static connection_t *_connection_new_SSL(char *dsthostname, char *dstport,
 
 connection_t *connection_new(char *dsthostname, int dstport, char *srchostname,
 		int srcport, int ssl, int ssl_check_mode, char *ssl_check_store,
-		int timeout)
+		char *ssl_client_certfile, int timeout)
 {
 	char dstportbuf[20], srcportbuf[20], *tmp;
 #ifndef HAVE_LIBSSL
@@ -1383,11 +1424,11 @@ connection_t *connection_new(char *dsthostname, int dstport, char *srchostname,
 		tmp = srcportbuf;
 	} else
 		tmp = NULL;
-	
 #ifdef HAVE_LIBSSL
 	if (ssl)
 		return _connection_new_SSL(dsthostname, dstportbuf, srchostname,
-				tmp, ssl_check_mode, ssl_check_store, timeout);
+				tmp, ssl_check_mode, ssl_check_store,
+				ssl_client_certfile, timeout);
 	else
 #endif
 		return _connection_new(dsthostname, dstportbuf, srchostname,
@@ -1424,7 +1465,15 @@ static int connection_timedout(connection_t *cn)
 
 static int socket_set_nonblock(int s)
 {
-	if (fcntl(s, F_SETFL, O_NONBLOCK) < 0) {
+	int flags;
+
+	if ((flags = fcntl(s, F_GETFL, 0)) < 0) {
+		mylog(LOG_ERROR, "Cannot set socket %d to non blocking : %s",
+				s, strerror(errno));
+		return 0;
+	}
+
+	if (fcntl(s, F_SETFL, flags | O_NONBLOCK) < 0) {
 		mylog(LOG_ERROR, "Cannot set socket %d to non blocking : %s",
 				s, strerror(errno));
 		return 0;
@@ -1461,7 +1510,7 @@ int main(int argc,char* argv[])
 	}
 	while (1) {
 		int ret = read_socket(conn2);
-		mylog(LOG_DEBUG, "READ: %d %*s",ret, conn2->incoming,
+		mylog(LOG_DEBUGTOOMUCH, "READ: %d %*s",ret, conn2->incoming,
 				conn2->incoming_end);
 		conn2->incoming_end = 0;
 		sleep(1);
