@@ -2030,8 +2030,8 @@ void irc_server_free(struct link_server *s)
 			hash_it_next(&hi)) {
 		struct channel *chan = hash_it_item(&hi);
 		channel_free(chan);
-
 	}
+	hash_clean(&s->channels);
 
 	free(s);
 }
@@ -2055,7 +2055,7 @@ connection_t *irc_server_connect(struct link *link)
 				link->user->ssl_check_store,
 				link->user->ssl_client_certfile,
 #else
-				0, 0, NULL,
+				0, 0, NULL, NULL,
 #endif
 				CONNECT_TIMEOUT);
 	if (!conn)
