@@ -2,7 +2,7 @@
  * $Id: irc.h,v 1.43 2005/04/21 06:58:50 nohar Exp $
  *
  * This file is part of the bip project
- * Copyright (C) 2004 2005 Arnaud Cornet and Loc Gomez
+ * Copyright (C) 2004 2005 Arnaud Cornet and Loïc Gomez
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -62,20 +62,63 @@ struct channel {
 #define IRC_TYPE_LOGING_CLIENT (2)
 #define IRC_TYPE_TRUST_CLIENT (3)
 
+struct user {
+	/** client connection static data **/
+
+	char *name;
+	unsigned char *password;
+	unsigned int seed;
+	int admin;
+	int bip_use_notice;
+
+	/* common link options */
+
+	char *default_nick;
+	char *default_username;
+	char *default_realname;
+
+	/* backlog options */
+	int backlog;
+	int backlog_lines;
+	int always_backlog;
+	int bl_msg_only;
+	int backlog_no_timestamp;
+	int blreset_on_talk;
+
+#ifdef HAVE_LIBSSL
+	int ssl_check_mode;
+	char *ssl_check_store;
+	char *ssl_client_certfile;
+#endif
+
+	hash_t connections;
+	int in_use; /* for mark and sweep on reload */
+};
+
+struct network
+{
+	char *name;
+#ifdef HAVE_LIBSSL
+	int ssl;
+#endif
+	int serverc;
+	struct server *serverv;
+};
+
 struct link {
-	char *name; 	/* id */
+	char *name;	/* id */
 
 	/** link live data **/
 	struct link_server *l_server;
 	int l_clientc;
 	struct link_client **l_clientv;
 
-	/* we honnor the /who from clients one client at a time, this is the 
+	struct log *log;
+
+	/* we honnor the /who from clients one client at a time, this is the
 	 * client that is /who-ing. Now for bans too */
 	struct link_client *who_client;
 
-	struct log *log;
-
 	/* server related live stuff */
 	int s_state;
 	int s_conn_attempt;
@@ -85,7 +128,7 @@ struct link {
 
 	/* connection state (reconnecting, was_connected ...) */
 	int recon_timer;
-	time_t last_connection;
+	time_t last_connection_attempt;
 
 	/** link options */
 
@@ -97,33 +140,27 @@ struct link {
 	hash_t chan_infos;		/* channels we want */
 	list_t chan_infos_order;	/* for order only */
 
-	/** client connection static data **/
-
-	char *username;
-	unsigned char *password;
-	unsigned int seed;
+	struct user *user;
 
 	/** server connection static data **/
 	/* server list */
-	int serverc;
-	struct server **serverv;
+	struct network *network;
 	int cur_server;
 
-	char *user;
-	char *real_name;
+	char *username;
+	char *realname;
 	char *s_password;
 	char *connect_nick;
 
 	/* socket creation info */
 	char *vhost;
 	int bind_port;
-	int s_ssl;
 
 #ifdef HAVE_LIBSSL
 	int ssl_check_mode;
-	char *ssl_check_store;
 	STACK_OF(X509) *untrusted_certs;
 #endif
+	int in_use; /* for mark and sweep on reload */
 };
 
 struct link_connection {
@@ -168,12 +205,14 @@ struct link_client {
 #define IRCS_CONNECTED (2)
 #define IRCS_WAS_CONNECTED (3)
 #define IRCS_RECONNECTING (4)
+#define IRCS_TIMER_WAIT (5)
 
 struct log;
 
 struct chan_info {
 	char *name;
 	char *key;
+	int backlog;
 };
 
 #define chan_info_new() calloc(sizeof(struct chan_info), 1)
@@ -195,14 +234,36 @@ struct link_server {
 	int lagtest_timeout;
 };
 
+typedef struct bip {
+	connection_t *listener;
+	/* all connected tcp connections */
+	list_t conn_list;
+	/* all links */
+	list_t link_list;
+	/* connecting clients */
+	list_t connecting_client_list;
+
+	hash_t networks;
+	hash_t users;
+	list_t errors;
+	struct link_client *reloading_client;
+
+#ifdef HAVE_OIDENTD
+	char *oidentdpath;
+#endif
+} bip_t;
+
+void bip_init(bip_t *bip);
 struct link_client *irc_client_new(void);
 struct link_server *irc_server_new(struct link *link, connection_t *conn);
 void irc_server_free(struct link_server *is);
 struct client *client_new();
-void irc_main(connection_t *inc, list_t *clientl);
+void irc_main(bip_t *);
 int ischannel(char p);
 void irc_client_close(struct link_client *);
 void irc_client_free(struct link_client *);
 struct link *irc_link_new();
+void link_kill(bip_t *bip, struct link *);
 void unbind_from_link(struct link_client *ic);
 #endif
+
