@@ -2,7 +2,7 @@
  * $Id: util.h,v 1.35 2005/04/12 19:34:35 nohar Exp $
  *
  * This file is part of the bip project
- * Copyright (C) 2004 2005 Arnaud Cornet and Loc Gomez
+ * Copyright (C) 2004 2005 Arnaud Cornet and Loïc Gomez
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,9 +15,9 @@
 #include <time.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <stdarg.h>
 
 /* Warning: must be in order, 0 = less output */
-#define LOG_STD -1
 #define LOG_FATAL 0
 #define LOG_ERROR 1
 #define LOG_WARN 2
@@ -30,9 +30,11 @@
 #define HASH_DEFAULT 0
 
 void mylog(int level, char *fmt, ...);
+void _mylog(int level, char *fmt, va_list ap);
 void fatal(char *fmt, ...);
 char *timestamp(void);
 struct list_item;
+struct hash_item;
 
 typedef struct list {
 	struct list_item *first;
@@ -43,19 +45,36 @@ typedef struct list {
 typedef struct list_iterator {
 	list_t *list;
 	struct list_item *cur;
+	struct list_item *next;
 } list_iterator_t;
 
-/* our hash is also a list */
 typedef struct hash {
 	list_t lists[256];
 } hash_t;
 
 typedef struct hash_iterator {
 	int list;
-	struct list_item *cur;
+	list_iterator_t lit;
+	struct hash_item *cur;
 	hash_t *hash;
 } hash_iterator_t;
 
+#define MOVE_STRING(dest, src) do {\
+	if (dest)\
+		free(dest);\
+	(dest) = (src);\
+	(src) = NULL;\
+} while(0)
+
+#define FREE(a) free(a); (a) = NULL;
+
+#define MAYFREE(a) do { \
+		if (a) { \
+			free(a); \
+			(a) = NULL; \
+		} \
+	} while(0);
+
 void list_init(list_t *list, int (*cmp)(void*,void*));
 int list_ptr_cmp(void *a, void *b);
 list_t *list_new(int (*cmp)(void *, void *));
@@ -86,10 +105,12 @@ void hash_insert(hash_t *hash, char *key, void *ptr);
 void *hash_get(hash_t *, char *key);
 void *hash_remove(hash_t *hash, char *key);
 void *hash_remove_if_exists(hash_t *hash, char *key);
+int hash_is_empty(hash_t *h);
 void hash_it_init(hash_t *hash, hash_iterator_t *i);
 void hash_it_next(hash_iterator_t *hi);
 void *hash_it_item(hash_iterator_t *h);
 char *hash_it_key(hash_iterator_t *h);
+void *hash_it_remove(hash_iterator_t *li);
 
 int is_valid_nick(char *str);
 int is_valid_username(char *str);
@@ -97,4 +118,10 @@ char *strmaydup(char *s);
 
 void strucase(char *s);
 int ischannel(char p);
+char *hrtime(time_t t);
+#ifdef HAVE_LIBSSL
+char *checkmode2text(int v);
+#endif
+#define bool2text(v) ((v) ? "true" : "false")
+
 #endif
